Package: l0ara
Type: Package
Title: Sparse Generalized Linear Model with L0 Approximation for
        Feature Selection
Version: 0.1.7
Date: 2026-04-25
Authors@R: c(
    person("Wenchuan", "Guo", email = "wguo1017@gmail.com", role = c("aut", "cre")),
    person("Shujie", "Ma", role = "aut"),
    person("Zhenqiu", "Liu", role = "aut")
    )
Description: Fits sparse generalized linear models using an adaptive ridge
    approximation to an L0 penalty. Supported model families include
    Gaussian, logistic, Poisson, gamma, and inverse Gaussian regression.
    The package also provides cross-validation for selecting the penalty
    parameter.
License: GPL-2
Imports: Rcpp (>= 0.12.6)
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-26 01:59:52 UTC; wguo
Author: Wenchuan Guo [aut, cre],
  Shujie Ma [aut],
  Zhenqiu Liu [aut]
Maintainer: Wenchuan Guo <wguo1017@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 07:20:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 17:13:49 UTC; unix
Archs: l0ara.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,cv.l0ara)
S3method(coef,l0ara)
S3method(plot,cv.l0ara)
S3method(plot,l0ara)
S3method(predict,l0ara)
S3method(print,cv.l0ara)
S3method(print,l0ara)
export(cv.l0ara)
export(l0ara)
import(graphics)
import(stats)
importFrom(Rcpp,evalCpp)
useDynLib(l0ara)

l0ara
=====

[![](http://www.r-pkg.org/badges/version/l0ara)](https://cran.r-project.org/package=l0ara)
[![CRAN RStudio mirror downloads](http://cranlogs.r-pkg.org/badges/l0ara)](https://cran.r-project.org/package=l0ara)

### Overview

`l0ara` fits sparse generalized linear models using an adaptive ridge
approximation to an L0 penalty. 

### Installation

Install the package from `CRAN` with:

```r
install.packages("l0ara")
```

### Basic usage

Fit a sparse Gaussian model:

```r
library(l0ara)

n <- 100
p <- 40
x <- matrix(rnorm(n * p), n, p)
beta <- c(1, 0, 2, 3, rep(0, p - 4))
y <- x %*% beta + rnorm(n)

fit <- l0ara(x, y, family = "gaussian", lam = log(n))
print(fit)
coef(fit)
```

Select the penalty by cross-validation:

```r
lam <- c(0.1, 0.3, 0.5)
cv_fit <- cv.l0ara(x, y, family = "gaussian", lam = lam, measure = "mse")

print(cv_fit)
coef(cv_fit)
plot(cv_fit)
```

#' Cross-validation for l0ara
#' @description Perform k-fold cross-validation over a supplied sequence of
#'   penalty values and return the value selected by the chosen measure.
#' @usage  cv.l0ara(x, y, family, lam, measure, nfolds, maxit, eps, seed)
#' @param x Input matrix as in \code{l0ara}.
#' @param y Response variable as in \code{l0ara}.
#' @param family Model family as in \code{l0ara}.
#' @param lam A user-supplied sequence of candidate penalty values. At least two
#'   values are required.
#' @param measure Criterion used to compare folds. Use \code{"mse"} or
#'   \code{"mae"} for any supported family. Use \code{"class"} or
#'   \code{"auc"} only with \code{family = "logit"}.
#' @param nfolds Number of folds. Default value is 10. Smallest value is 3.
#' @param maxit Maximum number of iterations passed to each call to
#'   \code{l0ara()}.
#' @param eps Convergence threshold. Default value is \code{1e-4}.
#' @param seed Optional random seed used to generate the fold assignments.
#' @details For each fold, the function fits one model per value in
#'   \code{lam}, evaluates the requested measure on the held-out data, and then
#'   averages the results across folds. For \code{measure = "auc"}, the selected
#'   value is the one with the largest score; for all other measures it is the
#'   one with the smallest score.
#' @return An object with S3 class "cv.l0ara" containing:
#'  \item{cv.error}{Mean cross-validation score for each value in
#'    \code{lambda}.}
#'  \item{cv.std}{Estimated standard error of \code{cv.error}.}
#'  \item{lam.min}{The selected penalty value.}
#'  \item{lambda}{The supplied sequence of penalty values.}
#'  \item{measure}{The measure used for model selection.}
#'  \item{family}{The fitted model family.}
#'  \item{x}{The original design matrix.}
#'  \item{y}{The original response vector.}
#'  \item{name}{A printable label for \code{measure}.}
#'  \item{fit.min}{A \code{"l0ara"} object refit on the full data using
#'    \code{lam.min}.}
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{l0ara}}, \code{\link{coef.cv.l0ara}},
#'   \code{\link{plot.cv.l0ara}}.
#' @examples
#' # Linear regression
#' # Generate design matrix and response variable
#' n <- 100
#' p <- 40
#' x <- matrix(rnorm(n*p), n, p)
#' beta <- c(1,0,2,3,rep(0,p-4))
#' noise <- rnorm(n)
#' y <- x%*%beta+noise
#' lam <- c(0.1, 0.3, 0.5)
#' fit <- cv.l0ara(x, y, family="gaussian", lam, measure = "mse")
#' @export

cv.l0ara <- function(x, y, family = c("gaussian", "logit", "gamma", "poisson", "inv.gaussian"), lam, measure = c("mse", "mae","class", "auc"), nfolds = 10,  maxit = 10^3, eps = 1e-04, seed){
  family <- match.arg(family)
  measure <- match.arg(measure)

  x <- .l0ara_as_matrix(x)
  y <- .l0ara_as_numeric(y)

  if (!is.null(lam) && length(lam) < 2) {
    stop("Need a sequence values for lambda", call. = FALSE)
  }
  if (nfolds < 3) {
    stop("nfolds must be bigger than 3", call. = FALSE)
  }
  if (!is.null(dim(x)) && length(y) != nrow(x)) {
    stop("length of y not equal to the number of rows of x", call. = FALSE)
  }
  if (!missing(seed)) {
    set.seed(seed)
  }
  if ((measure == "class" || measure == "auc") && family != "logit") {
    stop(sprintf("type='%s' can only be used with family='logit'", measure), call. = FALSE)
  }

  n <- nrow(x)
  p <- ncol(x)
  nlam <- length(lam)
  error <- matrix(NA_real_, nrow = nlam, ncol = nfolds)
  fold_id <- sample(rep(seq_len(nfolds), length.out = n))

  for (i in seq_len(nfolds)) {
    test_idx <- fold_id == i
    train_x <- x[!test_idx, , drop = FALSE]
    train_y <- y[!test_idx]
    test_x <- x[test_idx, , drop = FALSE]
    test_y <- y[test_idx]

    beta_matrix <- matrix(0, nrow = p, ncol = nlam)
    for (j in seq_len(nlam)) {
      beta_matrix[, j] <- .l0ara_fit_beta(
        x = train_x,
        y = train_y,
        family = family,
        lam = lam[j],
        maxit = maxit,
        eps = eps
      )
    }
    eta <- test_x %*% beta_matrix

    if (measure == "mse") {
      pred <- .l0ara_family_response(eta, family)
      error[, i] <- colMeans((pred - test_y) ^ 2)
    } else if (measure == "mae") {
      pred <- .l0ara_family_response(eta, family)
      error[, i] <- colMeans(abs(pred - test_y))
    } else if (measure == "class") {
      pred <- eta > 0
      error[, i] <- colMeans(pred != test_y)
    } else if (measure == "auc") {
      pred <- eta > 0
      neg_idx <- test_y == 0
      pos_idx <- test_y == 1
      fp <- colMeans(pred[neg_idx, , drop = FALSE])
      tp <- colMeans(pred[pos_idx, , drop = FALSE])
      error[, i] <- (tp - fp + 1) / 2
    }
  }

  cv.error <- rowMeans(error)
  centered <- error - cv.error
  cv.std <- sqrt(rowSums(centered * centered) / (nfolds - 1)) / sqrt(n)
  lam.min <- if (measure == "auc") lam[which.max(cv.error)] else lam[which.min(cv.error)]
  out <- list(
    cv.error = cv.error,
    cv.std = cv.std,
    lam.min = lam.min,
    measure = measure,
    lambda = lam,
    family = family,
    x = x,
    y = y,
    name = .l0ara_measure_label(measure),
    fit.min = structure(
      .l0ara_fit_raw(
        x = x,
        y = y,
        family = family,
        lam = lam.min,
        maxit = maxit,
        eps = eps
      ),
      class = "l0ara"
    )
  )
  class(out) <- "cv.l0ara"
  out
}

#' Make predictions from a \code{"l0ara"} object
#' @description Generate linear predictors, response-scale predictions, class
#'   labels, or fitted coefficients from a \code{"l0ara"} fit.
#' @param object Fitted "l0ara" object.
#' @param newx Matrix of new predictor values. If omitted, predictions are made
#'   for the training design matrix stored in \code{object$x}.
#' @param type Type of prediction required. \code{"link"} returns the linear
#'   predictor. \code{"response"} returns the response-scale mean implied by the
#'   model family. \code{"coefficients"} returns the fitted coefficient vector.
#'   \code{"class"} is available only for \code{family = "logit"} and returns
#'   0/1 class labels using a cutoff of 0 on the linear predictor.
#' @param ... Not used argument.
#' @details For Gaussian models, \code{type = "link"} and
#'   \code{type = "response"} return the same values.
#' @return The return value depends on \code{type}: a numeric vector of linear
#'   predictors or response-scale predictions, a named coefficient vector, or a
#'   numeric vector of 0/1 class labels.
#' @author
#' Wenchuan Guo <wguo007@ucr.edu>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{coef.l0ara}}, \code{\link{l0ara}}.
#' @export
predict.l0ara <- function(object, newx, type=c("link", "response", "coefficients", "class"), ...){
  type <- match.arg(type)
  beta <- coef.l0ara(object)
  if (missing(newx)) {
    newx <- object$x
  } else {
    newx <- .l0ara_as_matrix(newx)
  }
  if (type == "coefficients") {
    return(beta)
  }
  eta <- .l0ara_linear_predictor(beta, newx)
  if (type == "link" || object$family == "gaussian") {
    return(eta)
  }
  response <- drop(.l0ara_family_response(eta, object$family))
  if (type == "response") {
    return(response)
  }
  if (type=="class") {
    if (object$family=="logit") {
      return(drop(1 * (eta > 0)))
    } else {
      stop("type='class' can only be used with family='logit'", call. = FALSE)
    }
  }
}

#' Extract coefficients from a \code{"l0ara"} object
#' @description Return the fitted coefficient vector.
#' @param object Fitted "l0ara" object.
#' @param ... Not used argument.
#' @return A named numeric vector of fitted coefficients.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{predict.l0ara}}, \code{\link{l0ara}}.
#' @export
coef.l0ara <- function(object, ...){
  coefs <- as.vector(object$beta)
  names(coefs) <- .l0ara_coef_names(object)
  coefs
}

#' Extract coefficients from a \code{"cv.l0ara"} object
#' @description Return coefficients from the model refit at the selected
#'   \code{lam.min}.
#' @param object Fitted "cv.l0ara" object.
#' @param ... Not used argument.
#' @details If \code{object$fit.min} is missing, the model is refit on the full
#'   data using \code{lam.min}.
#' @return A named numeric vector of fitted coefficients.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{predict.l0ara}}, \code{\link{l0ara}},
#'   \code{\link{cv.l0ara}}.
#' @export
coef.cv.l0ara <- function(object, ...){
  fit <- object$fit.min
  if (is.null(fit)) {
    fit <- l0ara(object$x, object$y, object$family, object$lam.min)
  }
  coefs <- as.vector(fit$beta)
  names(coefs) <- .l0ara_coef_names(fit)
  coefs
}


#' Print a \code{"l0ara"} object
#' @description Print a short summary of a fitted model.
#' @param x Fitted "l0ara" object.
#' @param ... Not used argument.
#' @return Called for its side effect of printing model information.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{predict.l0ara}}, \code{\link{coef.l0ara}},
#'   \code{\link{l0ara}}.
#' @export
print.l0ara <- function(x, ...){
  family <- .l0ara_family_label(x$family)
  cat("Lambda used : ", .l0ara_lambda_value(x), "\n")
  cat("Model : ", family, "\n")
  cat("Iterations : ", x$iter, "\n")
  cat("Degree of freedom : ", x$df,"\n")
}

#' Print a \code{"cv.l0ara"} object
#' @description Print a short summary of a cross-validated fit.
#' @param x Fitted "cv.l0ara" object.
#' @param ... Not used argument.
#' @return Called for its side effect of printing cross-validation information.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{coef.cv.l0ara}}, \code{\link{plot.cv.l0ara}},
#'   \code{\link{cv.l0ara}}, \code{\link{l0ara}}.
#' @export
print.cv.l0ara <- function(x, ...){
  measure <- .l0ara_measure_label(x$measure)
  family <- .l0ara_family_label(x$family)
  cat("Number of Lambda used : ", length(x$lambda), "\n")
  cat("Optimal Lambda : ", x$lam.min, "\n")
  cat("Model : ", family, "\n")
  cat("Measure : ", measure, "\n")
  cat("Minimumn error : ",min(x$cv.error), "\n")
}

#' Plot a \code{"l0ara"} object
#' @description Plot fitted values against the linear predictor and, for
#'   logistic models, optionally add an ROC curve based on in-sample fitted
#'   probabilities.
#' @param x Fitted "l0ara" object.
#' @param auc Logical; if \code{TRUE} and \code{x$family == "logit"}, also draw
#'   an ROC curve.
#' @param split Logical; if \code{TRUE}, do not change the graphics layout with
#'   \code{par(mfrow = ...)}.
#' @param col Color used for observed points and the ROC line.
#' @param ... Not used argument.
#' @return Called for its side effect of producing one or two plots.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{predict.l0ara}}, \code{\link{coef.l0ara}},
#'   \code{\link{l0ara}}.
#' @export
plot.l0ara <- function(x, auc = FALSE, split = FALSE, col = 4, ...){
  nplots <- if (auc) 2 else 1
  if (!split) {
    par(mfrow = c(1, nplots))
  }
  lp <- predict(x, type = "link")
  resp <- drop(.l0ara_family_response(lp, x$family))
  plot(lp, resp, xlab = "Linear predictor", ylab = "Fitted value", pch = 20, main = "Linear predictor v.s. Fitted", ...)
  points(lp, x$y, col = col, pch = 20)
  legend("bottomright", legend = c("Fitted", "Truth"), col = c(1, col), pch = rep(20, 2))

  if (x$family == "logit" && auc) {
    n.thres <- 50
    thres <- seq(0, 1, length.out = n.thres)
    pred <- outer(resp, thres, `>`)
    fp <- colMeans(pred[x$y == 0, , drop = FALSE] == 1)
    tp <- colMeans(pred[x$y == 1, , drop = FALSE] == 1)
    area <- (tp - fp + 1) / 2
    plot(
      NA,
      xlab = "False positive rate",
      ylab = "True positive rate",
      xlim = c(0, 1),
      ylim = c(0, 1),
      main = paste("ROC curve for lambda =", .l0ara_lambda_value(x)),
      sub = paste("max auc = ", round(max(area), 2), ";", "best cut-off = ", round(thres[which.max(area)], 2)),
      ...
    )
    lines(fp, tp, pch = 19, col = col, type = "b", lty = 2)
  }
}

#' Plot a \code{"cv.l0ara"} object
#' @description Plot the cross-validation scores against the supplied penalty
#'   values and mark the selected \code{lam.min}.
#' @param x Fitted "cv.l0ara" object.
#' @param col Color used for the plotted points.
#' @param ... Not used argument.
#' @return Called for its side effect of producing a plot.
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{coef.cv.l0ara}}, \code{\link{cv.l0ara}},
#'   \code{\link{l0ara}}.
#' @export

plot.cv.l0ara <- function(x, col = 3, ...) {
  cv.u <- x$cv.error + x$cv.std
  cv.l <- x$cv.error - x$cv.std
  plot(x = x$lambda, y = x$cv.error, main = .l0ara_family_label(x$family), ylim = range(cv.l, cv.u), xlab = "Lambda", ylab = x$name, col = col, pch = 19, sub = paste("Optimal lambda = ", round(x$lam.min, 2)))
  abline(v = x$lam.min, lty = 3)
}

.l0ara_as_matrix <- function(x) {
  if (is.matrix(x)) {
    return(x)
  }

  x_matrix <- try(model.matrix(~0 + ., data = x), silent = TRUE)
  if (inherits(x_matrix, "try-error")) {
    stop("x must be a matrix or able to be coerced to a matrix", call. = FALSE)
  }

  x_matrix
}

.l0ara_as_numeric <- function(y) {
  if (is.numeric(y)) {
    return(drop(y))
  }

  y_numeric <- try(as.numeric(y), silent = TRUE)
  if (inherits(y_numeric, "try-error")) {
    stop("y must numeric or able to be coerced to numeric", call. = FALSE)
  }

  drop(y_numeric)
}

.l0ara_family_response <- function(eta, family) {
  switch(
    family,
    gaussian = eta,
    logit = plogis(eta),
    poisson = exp(eta),
    gamma = 1 / eta,
    inv.gaussian = 1 / sqrt(eta)
  )
}

.l0ara_family_label <- function(family) {
  switch(
    family,
    gaussian = "Linear regression",
    logit = "Logistic regression",
    poisson = "Poisson regression",
    inv.gaussian = "Inverse gaussian regression",
    gamma = "Gamma regression"
  )
}

.l0ara_measure_label <- function(measure) {
  switch(
    measure,
    mse = "Mean square error",
    mae = "Mean absolute error",
    class = "Misclassification rate",
    auc = "Area under the curve"
  )
}

.l0ara_lambda_value <- function(object) {
  if (!is.null(object$lam)) {
    return(object$lam)
  }

  object$lambda
}

.l0ara_coef_names <- function(object) {
  beta <- object$beta
  n_beta <- length(beta)
  coef_names <- character(n_beta)

  if (n_beta > 0L) {
    coef_names[1L] <- "Intercept"
  }
  if (n_beta > 1L) {
    coef_names[2L:n_beta] <- paste0("X", seq_len(n_beta - 1L))
  }

  coef_names
}

.l0ara_linear_predictor <- function(beta, newx) {
  drop(newx %*% beta)
}

.l0ara_fit_raw <- function(x, y, family, lam, maxit, eps, standardize = TRUE) {
  if (standardize) {
    xx <- scale(x)
  } else {
    xx <- x
  }

  out <- l0araC(xx, y, family, lam, maxit, eps)
  beta <- drop(out$beta)

  list(
    beta = beta,
    df = sum(beta != 0),
    lam = lam,
    lambda = lam,
    iter = out$iter,
    family = family,
    x = x,
    y = y
  )
}

.l0ara_fit_beta <- function(x, y, family, lam, maxit, eps, standardize = TRUE) {
  if (standardize) {
    xx <- scale(x)
  } else {
    xx <- x
  }

  out <- l0araC(xx, y, family, lam, maxit, eps)
  beta <- out$beta
  drop(beta)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

l0araC <- function(x, y, family, lam, maxit, eps) {
    .Call('_l0ara_l0araC', PACKAGE = 'l0ara', x, y, family, lam, maxit, eps)
}

#' Fit a generalized linear model with an L0 penalty
#' @description Fit a sparse generalized linear model by approximating the
#'   L0-penalized objective with an adaptive ridge algorithm.
#' @usage  l0ara(x, y, family, lam, standardize, maxit, eps)
#' @param x Design matrix. A numeric matrix is used as-is; other inputs must be
#'   coercible to a model matrix. Rows correspond to observations and columns to
#'   predictors.
#' @param y Response vector. For accurate use, supply numeric outcomes for
#'   \code{family = "gaussian"}, positive numeric outcomes for
#'   \code{family = "gamma"} and \code{family = "inv.gaussian"}, binary outcomes
#'   coded as 0/1 for \code{family = "logit"}, and non-negative counts for
#'   \code{family = "poisson"}.
#' @param family Model family.
#' @param lam A single user-supplied penalty value. If you have a candidate
#'   sequence of values, use \code{cv.l0ara()} to choose \code{lam.min} and then
#'   refit with \code{l0ara()}. Setting \code{lam = 2} mimics AIC-style model
#'   selection, and \code{lam = log(n)} mimics BIC-style model selection.
#' @param standardize Logical flag indicating whether to standardize the columns
#'   of \code{x} before fitting. The original \code{x} is stored in the returned
#'   object.
#' @param maxit Maximum number of iterations passed to the fitting routine.
#' @param eps Convergence threshold. Default value is \code{1e-4}.
#' @details The objective function is
#'   \deqn{-(\log\mbox{-}\mathrm{likelihood}) + (\lambda / 2)|\beta|_0,}
#'   where \eqn{|\beta|_0} is the number of non-zero elements of \eqn{\beta}.
#'   The adaptive ridge algorithm provides an efficient approximation to the
#'   corresponding L0-penalized generalized linear model.
#' @return An object with S3 class "l0ara" containing:
#'  \item{beta}{A vector of fitted coefficients.}
#'  \item{df}{Number of non-zero coefficients.}
#'  \item{iter}{Number of iterations used by the fitting routine.}
#'  \item{lam}{The supplied penalty value.}
#'  \item{lambda}{A copy of \code{lam} for compatibility with downstream
#'    methods.}
#'  \item{family}{The fitted model family.}
#'  \item{x}{The original design matrix supplied to the function.}
#'  \item{y}{The response vector supplied to the function.}
#' @author
#' Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
#' @seealso \code{\link{cv.l0ara}}, \code{\link{predict.l0ara}},
#'   \code{\link{coef.l0ara}}, \code{\link{plot.l0ara}}.
#' @useDynLib l0ara
#' @importFrom Rcpp evalCpp
#' @import stats
#' @import graphics
#' @examples
#' # Linear regression
#' # Generate design matrix and response variable
#' n <- 100
#' p <- 40
#' x <- matrix(rnorm(n*p), n, p)
#' beta <- c(1,0,2,3,rep(0,p-4))
#' noise <- rnorm(n)
#' y <- x%*%beta+noise
#' # fit sparse linear regression using BIC
#' res.gaussian <- l0ara(x, y, family="gaussian", log(n))
#'
#' # predict for new observations
#' print(res.gaussian)
#' predict(res.gaussian, newx=matrix(rnorm(3,p),3,p))
#' coef(res.gaussian)
#'
#' # Logistic regression
#' # Generate design matrix and response variable
#' n <- 100
#' p <- 40
#' x <- matrix(rnorm(n*p), n, p)
#' beta <- c(1,0,2,3,rep(0,p-4))
#' prob <- exp(x%*%beta)/(1+exp(x%*%beta))
#' y <- rbinom(n, rep(1, n), prob)
#' # fit sparse logistic regression
#' res.logit <- l0ara(x, y, family="logit", 0.7)
#'
#' # predict for new observations
#' print(res.logit)
#' predict(res.logit, newx=matrix(rnorm(3,p),3,p))
#' coef(res.logit)
#'
#' # Poisson regression
#' # Generate design matrix and response variable
#' n <- 100
#' p <- 40
#' x <- matrix(rnorm(n*p), n, p)
#' beta <- c(1,0,0.5,0.3,rep(0,p-4))
#' mu <- exp(x%*%beta)
#' y <- rpois(n, mu)
#' # fit sparse Poisson regression using AIC
#' res.pois <- l0ara(x, y, family="poisson", 2)
#'
#' # predict for new observations
#' print(res.pois)
#' predict(res.pois, newx=matrix(rnorm(3,p),3,p))
#' coef(res.pois)

#' @export
l0ara <- function(x, y, family = c("gaussian", "logit", "gamma", "poisson", "inv.gaussian"), lam, standardize = TRUE, maxit = 10^3, eps = 1e-04){
  x <- .l0ara_as_matrix(x)
  y <- .l0ara_as_numeric(y)

  family <- match.arg(family)
  if (missing(lam)) {
    stop("'lam' was not found(must be a postiive number)", call. = FALSE)
  }
  if (length(lam) > 1L) {
    stop("Require lenght >=1 for 'lam'", call. = FALSE)
  }
  if (lam < 0) {
    warning("lambda < 0; set to 0")
    lam <- 0
  }
  np <- dim(x)
  if (is.null(np) || np[2] <= 1L) {
    stop("x should be a matrix with 2 or more columns", call. = FALSE)
  }
  if (length(y) != np[1]) {
    stop("length of y not equal to the number of rows of x", call. = FALSE)
  }

  res <- .l0ara_fit_raw(
    x = x,
    y = y,
    family = family,
    lam = lam,
    maxit = maxit,
    eps = eps,
    standardize = standardize
  )
  class(res) <- "l0ara"
  res
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{coef.cv.l0ara}
\alias{coef.cv.l0ara}
\title{Extract coefficients from a \code{"cv.l0ara"} object}
\usage{
\method{coef}{cv.l0ara}(object, ...)
}
\arguments{
\item{object}{Fitted "cv.l0ara" object.}

\item{...}{Not used argument.}
}
\value{
A named numeric vector of fitted coefficients.
}
\description{
Return coefficients from the model refit at the selected
  \code{lam.min}.
}
\details{
If \code{object$fit.min} is missing, the model is refit on the full
  data using \code{lam.min}.
}
\seealso{
\code{\link{predict.l0ara}}, \code{\link{l0ara}},
  \code{\link{cv.l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.l0ara}
\alias{print.l0ara}
\title{Print a \code{"l0ara"} object}
\usage{
\method{print}{l0ara}(x, ...)
}
\arguments{
\item{x}{Fitted "l0ara" object.}

\item{...}{Not used argument.}
}
\value{
Called for its side effect of printing model information.
}
\description{
Print a short summary of a fitted model.
}
\seealso{
\code{\link{predict.l0ara}}, \code{\link{coef.l0ara}},
  \code{\link{l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{coef.l0ara}
\alias{coef.l0ara}
\title{Extract coefficients from a \code{"l0ara"} object}
\usage{
\method{coef}{l0ara}(object, ...)
}
\arguments{
\item{object}{Fitted "l0ara" object.}

\item{...}{Not used argument.}
}
\value{
A named numeric vector of fitted coefficients.
}
\description{
Return the fitted coefficient vector.
}
\seealso{
\code{\link{predict.l0ara}}, \code{\link{l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/l0ara.R
\name{l0ara}
\alias{l0ara}
\title{Fit a generalized linear model with an L0 penalty}
\usage{
l0ara(x, y, family, lam, standardize, maxit, eps)
}
\arguments{
\item{x}{Design matrix. A numeric matrix is used as-is; other inputs must be
coercible to a model matrix. Rows correspond to observations and columns to
predictors.}

\item{y}{Response vector. For accurate use, supply numeric outcomes for
\code{family = "gaussian"}, positive numeric outcomes for
\code{family = "gamma"} and \code{family = "inv.gaussian"}, binary outcomes
coded as 0/1 for \code{family = "logit"}, and non-negative counts for
\code{family = "poisson"}.}

\item{family}{Model family.}

\item{lam}{A single user-supplied penalty value. If you have a candidate
sequence of values, use \code{cv.l0ara()} to choose \code{lam.min} and then
refit with \code{l0ara()}. Setting \code{lam = 2} mimics AIC-style model
selection, and \code{lam = log(n)} mimics BIC-style model selection.}

\item{standardize}{Logical flag indicating whether to standardize the columns
of \code{x} before fitting. The original \code{x} is stored in the returned
object.}

\item{maxit}{Maximum number of iterations passed to the fitting routine.}

\item{eps}{Convergence threshold. Default value is \code{1e-4}.}
}
\value{
An object with S3 class "l0ara" containing:
 \item{beta}{A vector of fitted coefficients.}
 \item{df}{Number of non-zero coefficients.}
 \item{iter}{Number of iterations used by the fitting routine.}
 \item{lam}{The supplied penalty value.}
 \item{lambda}{A copy of \code{lam} for compatibility with downstream
   methods.}
 \item{family}{The fitted model family.}
 \item{x}{The original design matrix supplied to the function.}
 \item{y}{The response vector supplied to the function.}
}
\description{
Fit a sparse generalized linear model by approximating the
  L0-penalized objective with an adaptive ridge algorithm.
}
\details{
The objective function is
  \deqn{-(\log\mbox{-}\mathrm{likelihood}) + (\lambda / 2)|\beta|_0,}
  where \eqn{|\beta|_0} is the number of non-zero elements of \eqn{\beta}.
  The adaptive ridge algorithm provides an efficient approximation to the
  corresponding L0-penalized generalized linear model.
}
\examples{
# Linear regression
# Generate design matrix and response variable
n <- 100
p <- 40
x <- matrix(rnorm(n*p), n, p)
beta <- c(1,0,2,3,rep(0,p-4))
noise <- rnorm(n)
y <- x\%*\%beta+noise
# fit sparse linear regression using BIC
res.gaussian <- l0ara(x, y, family="gaussian", log(n))

# predict for new observations
print(res.gaussian)
predict(res.gaussian, newx=matrix(rnorm(3,p),3,p))
coef(res.gaussian)

# Logistic regression
# Generate design matrix and response variable
n <- 100
p <- 40
x <- matrix(rnorm(n*p), n, p)
beta <- c(1,0,2,3,rep(0,p-4))
prob <- exp(x\%*\%beta)/(1+exp(x\%*\%beta))
y <- rbinom(n, rep(1, n), prob)
# fit sparse logistic regression
res.logit <- l0ara(x, y, family="logit", 0.7)

# predict for new observations
print(res.logit)
predict(res.logit, newx=matrix(rnorm(3,p),3,p))
coef(res.logit)

# Poisson regression
# Generate design matrix and response variable
n <- 100
p <- 40
x <- matrix(rnorm(n*p), n, p)
beta <- c(1,0,0.5,0.3,rep(0,p-4))
mu <- exp(x\%*\%beta)
y <- rpois(n, mu)
# fit sparse Poisson regression using AIC
res.pois <- l0ara(x, y, family="poisson", 2)

# predict for new observations
print(res.pois)
predict(res.pois, newx=matrix(rnorm(3,p),3,p))
coef(res.pois)
}
\seealso{
\code{\link{cv.l0ara}}, \code{\link{predict.l0ara}},
  \code{\link{coef.l0ara}}, \code{\link{plot.l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{predict.l0ara}
\alias{predict.l0ara}
\title{Make predictions from a \code{"l0ara"} object}
\usage{
\method{predict}{l0ara}(
  object,
  newx,
  type = c("link", "response", "coefficients", "class"),
  ...
)
}
\arguments{
\item{object}{Fitted "l0ara" object.}

\item{newx}{Matrix of new predictor values. If omitted, predictions are made
for the training design matrix stored in \code{object$x}.}

\item{type}{Type of prediction required. \code{"link"} returns the linear
predictor. \code{"response"} returns the response-scale mean implied by the
model family. \code{"coefficients"} returns the fitted coefficient vector.
\code{"class"} is available only for \code{family = "logit"} and returns
0/1 class labels using a cutoff of 0 on the linear predictor.}

\item{...}{Not used argument.}
}
\value{
The return value depends on \code{type}: a numeric vector of linear
  predictors or response-scale predictions, a named coefficient vector, or a
  numeric vector of 0/1 class labels.
}
\description{
Generate linear predictors, response-scale predictions, class
  labels, or fitted coefficients from a \code{"l0ara"} fit.
}
\details{
For Gaussian models, \code{type = "link"} and
  \code{type = "response"} return the same values.
}
\seealso{
\code{\link{coef.l0ara}}, \code{\link{l0ara}}.
}
\author{
Wenchuan Guo <wguo007@ucr.edu>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{plot.cv.l0ara}
\alias{plot.cv.l0ara}
\title{Plot a \code{"cv.l0ara"} object}
\usage{
\method{plot}{cv.l0ara}(x, col = 3, ...)
}
\arguments{
\item{x}{Fitted "cv.l0ara" object.}

\item{col}{Color used for the plotted points.}

\item{...}{Not used argument.}
}
\value{
Called for its side effect of producing a plot.
}
\description{
Plot the cross-validation scores against the supplied penalty
  values and mark the selected \code{lam.min}.
}
\seealso{
\code{\link{coef.cv.l0ara}}, \code{\link{cv.l0ara}},
  \code{\link{l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{plot.l0ara}
\alias{plot.l0ara}
\title{Plot a \code{"l0ara"} object}
\usage{
\method{plot}{l0ara}(x, auc = FALSE, split = FALSE, col = 4, ...)
}
\arguments{
\item{x}{Fitted "l0ara" object.}

\item{auc}{Logical; if \code{TRUE} and \code{x$family == "logit"}, also draw
an ROC curve.}

\item{split}{Logical; if \code{TRUE}, do not change the graphics layout with
\code{par(mfrow = ...)}.}

\item{col}{Color used for observed points and the ROC line.}

\item{...}{Not used argument.}
}
\value{
Called for its side effect of producing one or two plots.
}
\description{
Plot fitted values against the linear predictor and, for
  logistic models, optionally add an ROC curve based on in-sample fitted
  probabilities.
}
\seealso{
\code{\link{predict.l0ara}}, \code{\link{coef.l0ara}},
  \code{\link{l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.l0ara.R
\name{cv.l0ara}
\alias{cv.l0ara}
\title{Cross-validation for l0ara}
\usage{
cv.l0ara(x, y, family, lam, measure, nfolds, maxit, eps, seed)
}
\arguments{
\item{x}{Input matrix as in \code{l0ara}.}

\item{y}{Response variable as in \code{l0ara}.}

\item{family}{Model family as in \code{l0ara}.}

\item{lam}{A user-supplied sequence of candidate penalty values. At least two
values are required.}

\item{measure}{Criterion used to compare folds. Use \code{"mse"} or
\code{"mae"} for any supported family. Use \code{"class"} or
\code{"auc"} only with \code{family = "logit"}.}

\item{nfolds}{Number of folds. Default value is 10. Smallest value is 3.}

\item{maxit}{Maximum number of iterations passed to each call to
\code{l0ara()}.}

\item{eps}{Convergence threshold. Default value is \code{1e-4}.}

\item{seed}{Optional random seed used to generate the fold assignments.}
}
\value{
An object with S3 class "cv.l0ara" containing:
 \item{cv.error}{Mean cross-validation score for each value in
   \code{lambda}.}
 \item{cv.std}{Estimated standard error of \code{cv.error}.}
 \item{lam.min}{The selected penalty value.}
 \item{lambda}{The supplied sequence of penalty values.}
 \item{measure}{The measure used for model selection.}
 \item{family}{The fitted model family.}
 \item{x}{The original design matrix.}
 \item{y}{The original response vector.}
 \item{name}{A printable label for \code{measure}.}
 \item{fit.min}{A \code{"l0ara"} object refit on the full data using
   \code{lam.min}.}
}
\description{
Perform k-fold cross-validation over a supplied sequence of
  penalty values and return the value selected by the chosen measure.
}
\details{
For each fold, the function fits one model per value in
  \code{lam}, evaluates the requested measure on the held-out data, and then
  averages the results across folds. For \code{measure = "auc"}, the selected
  value is the one with the largest score; for all other measures it is the
  one with the smallest score.
}
\examples{
# Linear regression
# Generate design matrix and response variable
n <- 100
p <- 40
x <- matrix(rnorm(n*p), n, p)
beta <- c(1,0,2,3,rep(0,p-4))
noise <- rnorm(n)
y <- x\%*\%beta+noise
lam <- c(0.1, 0.3, 0.5)
fit <- cv.l0ara(x, y, family="gaussian", lam, measure = "mse")
}
\seealso{
\code{\link{l0ara}}, \code{\link{coef.cv.l0ara}},
  \code{\link{plot.cv.l0ara}}.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Shujie Ma <shujie.ma@ucr.edu>, Zhenqiu Liu <Zhenqiu.Liu@cshs.org>
}
